@@ -1,6 +1,6 @@
 ;; c99x-act.scm
 
-;; Copyright (C) 2016-2018 Matthew R. Wette
+;; Copyright (C) 2016-2020 Matthew R. Wette
 ;; 
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -61,11 +61,9 @@
    ;; argument-expression-list => argument-expression-list "," arg-expr-hack
    (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
    ;; arg-expr-hack => declaration-specifiers abstract-declarator
-   (lambda ($2 $1 . $rest)
-     `(param-decl ,(tl->list $1) $2))
+   (lambda ($2 $1 . $rest) `(param-decl ,1 ,$2))
    ;; arg-expr-hack => declaration-specifiers
-   (lambda ($1 . $rest)
-     `(param-decl ,(tl->list $1)))
+   (lambda ($1 . $rest) `(param-decl ,$1))
    ;; unary-expression => postfix-expression
    (lambda ($1 . $rest) $1)
    ;; unary-expression => "++" unary-expression
@@ -362,8 +360,8 @@
    ;; fixpt-type-specifier => "signed" "_Fract"
    (lambda ($2 $1 . $rest)
      '(fixpt-type "signed _Fract"))
-   ;; fixpt-type-specifier => "signed" "long _Fract"
-   (lambda ($2 $1 . $rest)
+   ;; fixpt-type-specifier => "signed" "long" "_Fract"
+   (lambda ($3 $2 $1 . $rest)
      '(fixpt-type "signed long _Fract"))
    ;; fixpt-type-specifier => "unsigned" "short" "_Fract"
    (lambda ($3 $2 $1 . $rest)
@@ -371,16 +369,17 @@
    ;; fixpt-type-specifier => "unsigned" "_Fract"
    (lambda ($2 $1 . $rest)
      '(fixpt-type "unsigned _Fract"))
-   ;; fixpt-type-specifier => "unsigned" "long _Fract"
-   (lambda ($2 $1 . $rest)
+   ;; fixpt-type-specifier => "unsigned" "long" "_Fract"
+   (lambda ($3 $2 $1 . $rest)
      '(fixpt-type "unsigned long _Fract"))
    ;; fixpt-type-specifier => "short" "_Accum"
    (lambda ($2 $1 . $rest)
      '(fixpt-type "short _Accum"))
    ;; fixpt-type-specifier => "_Accum"
    (lambda ($1 . $rest) '(fixpt-type "_Accum"))
-   ;; fixpt-type-specifier => "long _Accum"
-   (lambda ($1 . $rest) '(fixpt-type "long _Accum"))
+   ;; fixpt-type-specifier => "long" "_Accum"
+   (lambda ($2 $1 . $rest)
+     '(fixpt-type "long _Accum"))
    ;; fixpt-type-specifier => "signed" "short" "_Accum"
    (lambda ($3 $2 $1 . $rest)
      '(fixpt-type "signd short _Accum"))
@@ -414,8 +413,8 @@
    ;; fixpt-type-specifier => "_Sat" "signed" "_Fract"
    (lambda ($3 $2 $1 . $rest)
      '(fixpt-type "_Sat signed _Fract"))
-   ;; fixpt-type-specifier => "_Sat" "signed" "long _Fract"
-   (lambda ($3 $2 $1 . $rest)
+   ;; fixpt-type-specifier => "_Sat" "signed" "long" "_Fract"
+   (lambda ($4 $3 $2 $1 . $rest)
      '(fixpt-type "_Sat signed long _Fract"))
    ;; fixpt-type-specifier => "_Sat" "unsigned" "short" "_Fract"
    (lambda ($4 $3 $2 $1 . $rest)
@@ -953,41 +952,51 @@
    ;; jump-statement => "return" ";"
    (lambda ($2 $1 . $rest) `(return (expr)))
    ;; asm-statement => asm-expression ";"
-   (lambda ($2 $1 . $rest) $1)
-   ;; asm-expression => "__asm__" opt-asm-specifiers "(" string-literal ")"
+   (lambda ($2 $1 . $rest) `(expr-stmt ,$1))
+   ;; asm-expression => "__asm__" opt-asm-qualifiers "(" string-literal ")"
    (lambda ($5 $4 $3 $2 $1 . $rest)
-     `(asm-expr (@ (extension "GNUC")) ,$4))
-   ;; asm-expression => "__asm__" opt-asm-specifiers "(" string-literal asm...
+     `(asm-expr (@ (extension "GNUC") ,@$2) ,$4))
+   ;; asm-expression => "__asm__" opt-asm-qualifiers "(" string-literal asm...
    (lambda ($6 $5 $4 $3 $2 $1 . $rest)
-     `(asm-expr
-        (@ (extension "GNUC"))
-        ,$4
-        ,(tl->list $5)))
-   ;; asm-expression => "__asm__" opt-asm-specifiers "(" string-literal asm...
+     `(asm-expr (@ (extension "GNUC") ,@$2) ,$4 ,$5))
+   ;; asm-expression => "__asm__" opt-asm-qualifiers "(" string-literal asm...
    (lambda ($7 $6 $5 $4 $3 $2 $1 . $rest)
      `(asm-expr
-        (@ (extension "GNUC"))
+        (@ (extension "GNUC") ,@$2)
         ,$4
-        ,(tl->list $5)
-        ,(tl->list $6)))
-   ;; asm-expression => "__asm__" opt-asm-specifiers "(" string-literal asm...
+        ,$5
+        ,$6))
+   ;; asm-expression => "__asm__" opt-asm-qualifiers "(" string-literal asm...
    (lambda ($8 $7 $6 $5 $4 $3 $2 $1 . $rest)
      `(asm-expr
-        (@ (extension "GNUC"))
+        (@ (extension "GNUC") ,@$2)
         ,$4
-        ,(tl->list $5)
-        ,(tl->list $6)
-        ,(tl->list $7)))
-   ;; opt-asm-specifiers => 
+        ,$5
+        ,$6
+        ,$7))
+   ;; asm-expression => "__asm__" opt-asm-qualifiers "(" string-literal asm...
+   (lambda ($9 $8 $7 $6 $5 $4 $3 $2 $1 . $rest)
+     `(asm-expr
+        (@ (extension "GNUC") ,@$2)
+        ,$4
+        (asm-outputs)
+        ,$6
+        ,$7
+        ,$8))
+   ;; opt-asm-qualifiers => 
    (lambda $rest (list))
-   ;; opt-asm-specifiers => "volatile"
-   (lambda ($1 . $rest) $1)
-   ;; asm-outputs => ":"
+   ;; opt-asm-qualifiers => "volatile"
+   (lambda ($1 . $rest) (list '(volatile "true")))
+   ;; opt-asm-qualifiers => "goto"
+   (lambda ($1 . $rest) (list '(goto "true")))
+   ;; asm-outputs => asm-outputs-1
+   (lambda ($1 . $rest) (tl->list $1))
+   ;; asm-outputs-1 => ":"
    (lambda ($1 . $rest) (make-tl 'asm-outputs))
-   ;; asm-outputs => ":" asm-output
+   ;; asm-outputs-1 => ":" asm-output
    (lambda ($2 $1 . $rest)
      (make-tl 'asm-outputs $2))
-   ;; asm-outputs => asm-outputs "," asm-output
+   ;; asm-outputs-1 => asm-outputs-1 "," asm-output
    (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
    ;; asm-output => string-literal "(" identifier ")"
    (lambda ($4 $3 $2 $1 . $rest)
@@ -995,11 +1004,13 @@
    ;; asm-output => "[" identifier "]" string-literal "(" identifier ")"
    (lambda ($7 $6 $5 $4 $3 $2 $1 . $rest)
      `(asm-operand ,$2 ,$4 ,$6))
-   ;; asm-inputs => ":"
+   ;; asm-inputs => asm-inputs-1
+   (lambda ($1 . $rest) (tl->list $1))
+   ;; asm-inputs-1 => ":"
    (lambda ($1 . $rest) (make-tl 'asm-inputs))
-   ;; asm-inputs => ":" asm-input
+   ;; asm-inputs-1 => ":" asm-input
    (lambda ($2 $1 . $rest) (make-tl 'asm-inputs $2))
-   ;; asm-inputs => asm-inputs "," asm-input
+   ;; asm-inputs-1 => asm-inputs-1 "," asm-input
    (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
    ;; asm-input => string-literal "(" expression ")"
    (lambda ($4 $3 $2 $1 . $rest)
@@ -1007,14 +1018,23 @@
    ;; asm-input => "[" identifier "]" string-literal "(" expression ")"
    (lambda ($7 $6 $5 $4 $3 $2 $1 . $rest)
      `(asm-operand ,$2 ,$4 ,$6))
-   ;; asm-clobbers => ":"
+   ;; asm-clobbers => asm-clobbers-1
+   (lambda ($1 . $rest) (tl->list $1))
+   ;; asm-clobbers-1 => ":"
    (lambda ($1 . $rest) (make-tl 'asm-clobbers))
-   ;; asm-clobbers => ":" string-literal
+   ;; asm-clobbers-1 => ":" string-literal
    (lambda ($2 $1 . $rest)
-     (tl-extend (make-tl 'asm-clobbers) $2))
-   ;; asm-clobbers => asm-clobbers "," string-literal
-   (lambda ($3 $2 $1 . $rest)
-     (tl-extend $1 (cdr $3)))
+     (make-tl 'asm-clobbers $2))
+   ;; asm-clobbers-1 => asm-clobbers-1 "," string-literal
+   (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
+   ;; asm-gotos => asm-gotos-1
+   (lambda ($1 . $rest) (tl->list $1))
+   ;; asm-gotos-1 => ":"
+   (lambda ($1 . $rest) (make-tl 'asm-gotos))
+   ;; asm-gotos-1 => ":" identifier
+   (lambda ($2 $1 . $rest) (make-tl 'asm-gotos $2))
+   ;; asm-gotos-1 => asm-gotos-1 "," identifier
+   (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
    ;; translation-unit => external-declaration-list
    (lambda ($1 . $rest) (tl->list $1))
    ;; external-declaration-list => 
