@@ -51,13 +51,13 @@
 ;; @end deffn
 (define c99-spec
   (lalr-spec
-   (notice (string-append "Copyright (C) 2016-2018 Matthew R. Wette"
+   (notice (string-append "Copyright (C) 2016-2020 Matthew R. Wette"
 			  license-lgpl3+))
 
    (prec< 'then "else")	       ; "then/else" SR-conflict resolution
-   (prec< 'imp		       ; "implied type" SR-conflict resolution
-	  (nonassoc "char" "short" "int" "long" "_Fract" "_Accum" "_Sat")
-	  (nonassoc "float" "double" "_Complex"))
+   (prec< (left 'imp)	       ; "implied type" SR-conflict resolution
+	  (left "char" "short" "int" "long" "_Fract" "_Accum" "_Sat")
+	  (left "float" "double" "_Complex"))
    (prec< 'shift-on-attr		; living on the edge ...
 	  (nonassoc "__attribute__" "__packed__" "__aligned__" "__alignof__")
 	  'reduce-on-attr
@@ -101,8 +101,8 @@
      (argument-expression-list "," arg-expr-hack ($$ (tl-append $1 $3))))
     (arg-expr-hack
      (declaration-specifiers
-      abstract-declarator ($$ `(param-decl ,(tl->list $1) $2)))
-     (declaration-specifiers ($$ `(param-decl ,(tl->list $1)))))
+      abstract-declarator ($$ `(param-decl ,1 ,$2)))
+     (declaration-specifiers ($$ `(param-decl ,$1))))
 
     (unary-expression
      (postfix-expression)		; S 6.5.3
@@ -308,13 +308,13 @@
      ("long" "_Fract" ($$ '(fixpt-type "long _Fract")))
      ("signed" "short" "_Fract" ($$ '(fixpt-type "signd short _Fract")))
      ("signed" "_Fract" ($$ '(fixpt-type "signed _Fract")))
-     ("signed" "long _Fract" ($$ '(fixpt-type "signed long _Fract")))
+     ("signed" "long" "_Fract" ($$ '(fixpt-type "signed long _Fract")))
      ("unsigned" "short" "_Fract" ($$ '(fixpt-type "unsigned short _Fract")))
      ("unsigned" "_Fract" ($$ '(fixpt-type "unsigned _Fract")))
-     ("unsigned" "long _Fract" ($$ '(fixpt-type "unsigned long _Fract")))
+     ("unsigned" "long" "_Fract" ($$ '(fixpt-type "unsigned long _Fract")))
      ("short" "_Accum" ($$ '(fixpt-type "short _Accum")))
      ("_Accum" ($$ '(fixpt-type "_Accum")))
-     ("long _Accum" ($$ '(fixpt-type "long _Accum")))
+     ("long" "_Accum" ($$ '(fixpt-type "long _Accum")))
      ("signed" "short" "_Accum" ($$ '(fixpt-type "signd short _Accum")))
      ("signed" "_Accum" ($$ '(fixpt-type "signed _Accum")))
      ("signed" "long" "_Accum" ($$ '(fixpt-type "signed long _Accum")))
@@ -327,7 +327,7 @@
      ("_Sat" "signed" "short" "_Fract"
       ($$ '(fixpt-type "_Sat signd short _Fract")))
      ("_Sat" "signed" "_Fract" ($$ '(fixpt-type "_Sat signed _Fract")))
-     ("_Sat" "signed" "long _Fract"
+     ("_Sat" "signed" "long" "_Fract"
       ($$ '(fixpt-type "_Sat signed long _Fract")))
      ("_Sat" "unsigned" "short" "_Fract"
       ($$ '(fixpt-type "_Sat unsigned short _Fract")))
@@ -753,41 +753,58 @@
      ("return" ";" ($$ `(return (expr)))))
 
     (asm-statement
-     (asm-expression ";"))
+     (asm-expression ";" ($$ `(expr-stmt ,$1))))
     (asm-expression
-     ("__asm__" opt-asm-specifiers "(" string-literal ")"
-      ($$ `(asm-expr (@ (extension "GNUC")) ,$4)))
-     ("__asm__" opt-asm-specifiers "(" string-literal asm-outputs ")"
-      ($$ `(asm-expr (@ (extension "GNUC")) ,$4 ,(tl->list $5))))
-     ("__asm__" opt-asm-specifiers "(" string-literal asm-outputs asm-inputs ")"
-      ($$ `(asm-expr (@ (extension "GNUC")) ,$4 ,(tl->list $5) ,(tl->list $6))))
-     ("__asm__" opt-asm-specifiers "(" string-literal asm-outputs
+     ("__asm__" opt-asm-qualifiers "(" string-literal ")"
+      ($$ `(asm-expr (@ (extension "GNUC") ,@$2) ,$4)))
+     ("__asm__" opt-asm-qualifiers "(" string-literal asm-outputs ")"
+      ($$ `(asm-expr (@ (extension "GNUC") ,@$2) ,$4 ,$5)))
+     ("__asm__" opt-asm-qualifiers "(" string-literal asm-outputs
+      asm-inputs ")"
+      ($$ `(asm-expr (@ (extension "GNUC") ,@$2) ,$4 ,$5 ,$6)))
+     ("__asm__" opt-asm-qualifiers "(" string-literal asm-outputs
       asm-inputs asm-clobbers ")"
-      ($$ `(asm-expr (@ (extension "GNUC"))
-		     ,$4 ,(tl->list $5) ,(tl->list $6) ,(tl->list $7)))))
-    (opt-asm-specifiers
+      ($$ `(asm-expr (@ (extension "GNUC") ,@$2) ,$4 ,$5 ,$6 ,$7)))
+     ("__asm__" opt-asm-qualifiers "(" string-literal asm-outputs
+      asm-inputs asm-clobbers asm-gotos ")"
+      ($$ `(asm-expr (@ (extension "GNUC") ,@$2) ,$4 (asm-outputs)
+		     ,$6 ,$7 ,$8))))
+    (opt-asm-qualifiers
      ($empty)
-     ("volatile"))
+     ("volatile" ($$ (list '(volatile "true"))))
+     ("goto" ($$ (list '(goto "true")))))
     (asm-outputs
+     (asm-outputs-1 ($$ (tl->list $1))))
+    (asm-outputs-1
      (":" ($$ (make-tl 'asm-outputs)))
      (":" asm-output ($$ (make-tl 'asm-outputs $2)))
-     (asm-outputs "," asm-output ($$ (tl-append $1 $3))))
+     (asm-outputs-1 "," asm-output ($$ (tl-append $1 $3))))
     (asm-output
      (string-literal "(" identifier ")" ($$ `(asm-operand ,$1 ,$3)))
      ("[" identifier "]" string-literal "(" identifier ")"
       ($$ `(asm-operand ,$2 ,$4 ,$6))))
     (asm-inputs
+     (asm-inputs-1 ($$ (tl->list $1))))
+    (asm-inputs-1
      (":" ($$ (make-tl 'asm-inputs)))
      (":" asm-input ($$ (make-tl 'asm-inputs $2)))
-     (asm-inputs "," asm-input ($$ (tl-append $1 $3))))
+     (asm-inputs-1 "," asm-input ($$ (tl-append $1 $3))))
     (asm-input
      (string-literal "(" expression ")" ($$ `(asm-operand ,$1 ,$3)))
      ("[" identifier "]" string-literal "(" expression ")"
       ($$ `(asm-operand ,$2 ,$4 ,$6))))
     (asm-clobbers
+     (asm-clobbers-1 ($$ (tl->list $1))))
+    (asm-clobbers-1
      (":" ($$ (make-tl 'asm-clobbers)))
-     (":" string-literal ($$ (tl-extend (make-tl 'asm-clobbers) $2)))
-     (asm-clobbers "," string-literal ($$ (tl-extend $1 (cdr $3)))))
+     (":" string-literal ($$ (make-tl 'asm-clobbers $2)))
+     (asm-clobbers-1 "," string-literal ($$ (tl-append $1 $3))))
+    (asm-gotos
+     (asm-gotos-1 ($$ (tl->list $1))))
+    (asm-gotos-1
+     (":" ($$ (make-tl 'asm-gotos)))
+     (":" identifier ($$ (make-tl 'asm-gotos $2)))
+     (asm-gotos-1 "," identifier ($$ (tl-append $1 $3))))
 
     ;; === top-level forms ====================================================
 
