@@ -59,9 +59,9 @@
   #:use-module (rnrs bytevectors)
   #:use-module ((system foreign) #:prefix ffi:)
   #:use-module (srfi srfi-9)
-  #:version (1 02 1))
+  #:version (1 03 1))
 
-(define *ffi-help-version* "1.02.1")
+(define *ffi-help-version* "1.03.1")
 
 (define (sferr fmt . args) (apply simple-format (current-error-port) fmt args))
 
@@ -653,12 +653,15 @@
    ((ffi:pointer? obj) obj)
    ((string? obj) (ffi:string->pointer obj))
    ((bytestructure? obj) (ffi:make-pointer (bytestructure-ref obj)))
+   ((bytevector? obj) (ffi:bytevector->pointer obj))
    ((fh-object? obj) (unwrap~pointer (struct-ref obj 0)))
    ;; TODO: work out casting pointer types
    ;;((and (pair? obj)
    ;;(or (fh-type? (car obj)) (bytestructure-descriptor? (car obj))))
    (else (error "expecting pointer type"))))
 
+(define unwrap~array unwrap~pointer)
+
 ;; @deffn {Procedure} make-fctn-param-unwrapper ret-t args-t => lambda
 ;; This procedure will convert an argument, 
 ;; @end deffn
@@ -864,6 +867,32 @@
     (make-bytestructure (ffi:pointer->bytevector addr size) 0 desc)))
 
 
+#|
+;; @deffn {Procedure} make-cstr-array str-list => bv
+;; For C functions that take an argument of the form @code{const char *names[]}, 
+;; this routine will convert a scheme list of strings into an appropriate
+;; bytevector which can be passed via @code{unwrap~pointer}.
+;; @end deffn
+(define (make-cstr-array str-list)
+  "- Procedure: make-cstr-array str-list => bv
+     For C functions that take an argument of the form 'const char
+     *names[]', this routine will convert a scheme list of strings into
+     an appropriate bytevector which can be passed via 'unwrap~pointer'."
+  (let* ((n (length string-list))
+         (addresses (map (compose pointer-address
+                                  string->pointer)
+                         string-list))
+	     (bv (make-bytevector (* n (sizeof '*))))
+	     (bv-set! (case (sizeof '*)
+		            ((4) bytevector-u32-native-set!)
+		            ((8) bytevector-u64-native-set!))))
+    (for-each (lambda (address index)
+                (bv-set! bv (* (sizeof '*) index) address))
+              addresses (iota n))
+    bv))
+(export make-cstr-array)
+|#
+
 ;; === common c functions called
 
 ;; @deffn {Procedure} fopen filename mode
